@@ -142,7 +142,7 @@ namespace xt
         const E2& de2 = e2.derived_cast();
 
         size_type dim = de2.dimension();
-        shape_type shape(dim, size_type(1));
+        shape_type shape = make_sequence<shape_type>(dim, size_type(1));
         bool trivial_broadcast = de2.broadcast_shape(shape);
 
         if (dim > de1.dimension() || shape > de1.shape())
@@ -173,7 +173,7 @@ namespace xt
         const E1& de1 = e1.derived_cast();
         const E2& de2 = e2.derived_cast();
         size_type size = de2.dimension();
-        shape_type shape(size, size_type(1));
+        shape_type shape = make_sequence<shape_type>(size, size_type(1));
         de2.broadcast_shape(shape);
         if (shape.size() > de1.shape().size() || shape > de1.shape())
         {
